#!/usr/bin/perl

use Statistics::OLS;
use Math::FFT;
use Math::Stat;
use Math::Spline qw(spline linsearch binsearch);
use Math::Derivative qw(Derivative2);
use Math::Approx;
use Astro::FITS::CFITSIO qw( :longnames :constants );
use PDL;
use PDL::Fit::Polynomial; 
use PDL::Filter::Linear;
use PGPLOT;  # Load PGPLOT module
use PDL::Fit::Gaussian;
use PDL::Slatec;
use PDL::Image2D;
#use PDL::Matrix;
use PDL::Func;
use PDL::Math;


$F3D=$ENV{FIT3D_PATH}; $F3D=$ENV{FIT3D_PATH}; require("$F3D/scripts/my.pl");


if ($#ARGV<1) {
    print "USE: Age_Met_rmodel.pl indices.INFILE rmodel.OUTPUT\n";
    $infile="indices.CS.NGC2916.rss.out";
    $outfile="rmodel.CS.NGC2916.rss.csv";
    print "Adopting $infile,$outfile\n";
#    exit;
} else {
    $infile=$ARGV[0];
    $outfile=$ARGV[1];
}




#print "IN $infile\n";

$n=0;
open(FH,"<$infile");
while($line=<FH>) {
    if ($line !~ "#") {
	chop($line);
	@data=split(" ",$line);
	for ($i=0;$i<($#data+1);$i=$i+3) {
	    $id=int($i/3);
	    $name[$id][$n]=$data[$i];
	    if ($name[$id][$n] ne "D4000") {
		$EW[$id][$n]=$data[$i+1];
		$e_EW[$id][$n]=$data[$i+2];
	    } else {
		$EW[$id][$n]=2.5*log10($data[$i+1]);
		$e_EW[$id][$n]=0.4*$data[$i+2]/$data[$i+1];
	    }
	    


#	    print "$id,$n,$name[$id][$n],$EW[$id][$n],$e_EW[$id][$n]\n";
	}

# [MgFe]
	$id=$id+1;
	$name[$id][$n]="[MgFe]";
	$EW[$id][$n]=sqrt(abs($EW[2][$n]*(0.72*$EW[3][$n]+0.28*$EW[4][$n])));
	$e_EW[$id][$n]=0.5*($e_EW[2][$n]*(0.72*$EW[3][$n]+0.28*$EW[4][$n])+$EW[2][$n]*(0.72*$e_EW[3][$n]+0.28*$e_EW[4][$n]))/$EW[$id][$n];
	$n++;
    }
}
close(FH);

#
# We create [MgFe] index
#



print "Determining Age and met...\n";
# Hb-Mg
print "Hb-Mg\n";
my ($pdl_age_HbMg,$pdl_e_age_HbMg,$pdl_met_HbMg,$pdl_e_met_HbMg)=Age_Met(1,2,9,13,0);
print "Done\n";
# Hb-Fe5270
print "Hb-Fe5270\n";
my ($pdl_age_HbFe5270,$pdl_e_age_HbFe5270,$pdl_met_HbFe5270,$pdl_e_met_HbFe5270)=Age_Met(1,3,9,14,0);
print "Done\n";
# Hb-Fe5270
print "Hb-Fe5335\n";
my ($pdl_age_HbFe5335,$pdl_e_age_HbFe5335,$pdl_met_HbFe5335,$pdl_e_met_HbFe5335)=Age_Met(1,4,9,15,0);
print "Done\n";

# D4000-Mg
print "D4000-Mg\n";
my ($pdl_age_D4000Mg,$pdl_e_age_D4000Mg,$pdl_met_D4000Mg,$pdl_e_met_D4000Mg)=Age_Met(5,2,30,13,1);
print "Done\n";
# D4000-Fe5270
print "D4000-Fe5270\n";
my ($pdl_age_D4000Fe5270,$pdl_e_age_D4000Fe5270,$pdl_met_D4000Fe5270,$pdl_e_met_D4000Fe5270)=Age_Met(5,3,30,14,1);
print "Done\n";
# D4000-Fe5270
print "D4000-Fe5335\n";
my ($pdl_age_D4000Fe5335,$pdl_e_age_D4000Fe5335,$pdl_met_D4000Fe5335,$pdl_e_met_D4000Fe5335)=Age_Met(5,4,30,15,1);
print "Done\n";

# Hb-[MgFe]
print "Hb-[MgFe]\n";
my ($pdl_age_HbMgFe,$pdl_e_age_HbMgFe,$pdl_met_HbMgFe,$pdl_e_met_HbMgFe)=Age_Met(1,9,9,18,0);
print "Done\n";
# D4000-[MgFe]
print "D4000-[MgFe]\n";
my ($pdl_age_D4000MgFe,$pdl_e_age_D4000MgFe,$pdl_met_D4000MgFe,$pdl_e_met_D4000MgFe)=Age_Met(5,9,30,18,1);
print "Done\n";

# D4000-Hb
print "D4000-Hb\n";
my ($pdl_age_D4000Hb,$pdl_e_age_D4000Hb,$pdl_met_D4000Hb,$pdl_e_met_D4000Hb)=Age_Met(5,1,30,9,1);
print "Done\n";
# Fe5270-Mg
print "Fe5270-Mg\n";
my ($pdl_age_Fe5270Mg,$pdl_e_age_Fe5270Mg,$pdl_met_Fe5270Mg,$pdl_e_met_Fe5270Mg)=Age_Met(3,2,14,13,0);
print "Done\n";

$pdl_age=pdl($pdl_age_HbMg,$pdl_age_HbFe5270,$pdl_age_HbFe5335,$pdl_age_D4000Mg,$pdl_age_D4000Fe5270,$pdl_age_D4000Fe5335,$pdl_age_HbMgFe,$pdl_age_D4000MgFe,$pdl_age_D4000Hb,$pdl_age_Fe5270Mg);

$pdl_met=pdl($pdl_met_HbMg,$pdl_met_HbFe5270,$pdl_met_HbFe5335,$pdl_met_D4000Mg,$pdl_met_D4000Fe5270,$pdl_met_D4000Fe5335,$pdl_met_HbMgFe,$pdl_met_D4000MgFe,$pdl_met_D4000Hb,$pdl_met_Fe5270Mg);

$pdl_e_age=pdl($pdl_e_age_HbMg,$pdl_e_age_HbFe5270,$pdl_e_age_HbFe5335,$pdl_e_age_D4000Mg,$pdl_e_age_D4000Fe5270,$pdl_e_age_D4000Fe5335,$pdl_e_age_HbMgFe,$pdl_e_age_D4000MgFe,$pdl_e_age_D4000Hb,$pdl_e_age_Fe5270Mg);

$pdl_e_met=pdl($pdl_e_met_HbMg,$pdl_e_met_HbFe5270,$pdl_e_met_HbFe5335,$pdl_e_met_D4000Mg,$pdl_e_met_D4000Fe5270,$pdl_e_met_D4000Fe5335,$pdl_e_met_HbMgFe,$pdl_e_met_D4000MgFe,$pdl_e_met_D4000Hb,$pdl_e_met_Fe5270Mg);


#$pdl_age->inplace->log10;
$pdl_e_age->inplace->setnantobad();
$pdl_e_age->inplace->setvaltobad(1000);
$pdl_e_met->inplace->setnantobad();
$pdl_e_met->inplace->setvaltobad(1000);

$pdl_e_met=$pdl_e_met*$pdl_e_met;
$pdl_e_age=$pdl_e_age*$pdl_e_age;

$pdl_age->inplace->setvaltobad(1000);
$pdl_met->inplace->setvaltobad(1000);
$pdl_age->inplace->setnantobad();
$pdl_met->inplace->setnantobad();


$pdl_age_med=zeroes($n);
$pdl_e_age_med=zeroes($n);
$pdl_met_med=zeroes($n);
$pdl_e_met_med=zeroes($n);

for ($i=0;$i<$n;$i++) {
    my @tmp_age;
    my @tmp_met;
    my @tmp_e_age;
    my @tmp_e_met;
    my $sum_w_age=0;
    my $sum_w_met=0;
    my $sum_age=0;
    my $sum_met=0;
    $min_error_age=0.2;
    $min_error_met=0.001;

    for ($j=0;$j<9;$j++) {
	$tmp_age[$j]=($pdl_age->at($i,$j));
	$tmp_met[$j]=($pdl_met->at($i,$j));
	$tmp_e_age[$j]=$pdl_e_age->at($i,$j);
	$tmp_e_met[$j]=$pdl_e_met->at($i,$j);
	$sum_age=$sum_age+$tmp_age[$j]/($min_error_age+abs($tmp_e_age[$j]))**2;
	$sum_met=$sum_met+$tmp_met[$j]/($min_error_met+abs($tmp_e_met[$j]))**2;
	$sum_w_age=$sum_w_age+1/($min_error_age+abs($tmp_e_age[$j]))**2;
	$sum_w_met=$sum_w_met+1/($min_error_met+abs($tmp_e_met[$j]))**2;
    }
    my $val_age=($sum_age/$sum_w_age);
    my $val_e_age=median(@tmp_e_age);
    my $val_met=($sum_met/$sum_w_met);
    my $val_e_met=median(@tmp_e_met);
#
# Second Loop!
#
    for ($j=0;$j<9;$j++) {
	$tmp_age[$j]=($pdl_age->at($i,$j));
	$tmp_met[$j]=($pdl_met->at($i,$j));
	$tmp_e_age[$j]=$pdl_e_age->at($i,$j);
	$tmp_e_met[$j]=$pdl_e_met->at($i,$j);
	$delta_age=($val_age-$tmp_age[$j])**2;
	$delta_met=($val_met-$tmp_met[$j])**2;
	$sum_age=$sum_age+$tmp_age[$j]/($delta_age+$min_error_age+abs($tmp_e_age[$j]))**2;
	$sum_met=$sum_met+$tmp_met[$j]/($delta_met+$min_error_met+abs($tmp_e_met[$j]))**2;
	$sum_w_age=$sum_w_age+1/($min_error_age+abs($tmp_e_age[$j]))**2;
	$sum_w_met=$sum_w_met+1/($min_error_met+abs($tmp_e_met[$j]))**2;
    }
    my $val_age=($sum_age/$sum_w_age);
    my $val_e_age=median(@tmp_e_age);
    my $val_met=($sum_met/$sum_w_met);
    my $val_e_met=median(@tmp_e_met);




    set($pdl_age_med,$i,$val_age);
    set($pdl_e_age_med,$i,$val_e_age);
    set($pdl_met_med,$i,$val_met);
    set($pdl_e_met_med,$i,$val_e_met);
}


#
# outputfile
#
open(OUT,">$outfile");
print OUT "# (1) ID\n";
print OUT "# (2) Age\n";
print OUT "# (3) Met\n";
for ($j=0;$j<$n;$j++) {
#    print "$j,$OUTPUT[$j][0]\n"; <stdin>;
#    $age_HbMg[$j]=$pdl_
    $age=$pdl_age_med->at($j);
    $e_age=$pdl_e_age_med->at($j);
    $met=$pdl_met_med->at($j);
    $e_met=$pdl_e_met_med->at($j);
    print "$j,$age,$e_age,$met,$e_met\n"; 
    print OUT "$j,$age,$e_age,$met,$e_met\n"; 
#    print OUT "$j,$age_HbMg[$j],$e_age_HbMg[$j],$met_HbMg[$j],$e_met_HbMg[$j]\n"; 
}
close(OUT);


exit;



sub Age_Met() {
    my $I0_index=$_[0]; # Hb is Input index
    my $I1_index=$_[1]; # Mg is Input index
    my $I0_rmodel=$_[2]; # Hb is 9
    my $I1_rmodel=$_[3]; # Mg is 13
    my $type=$_[4];

 #    print "PASO $n\n"; exit;

#
# Input indices file in the rmodel format
#
    my $tmpfile="tmp.indices";
    my $j;
    open(OUT,">$tmpfile");
    for ($j=0;$j<$n;$j++) {
	print OUT "$EW[$I0_index][$j] $e_EW[$I0_index][$j] $EW[$I1_index][$j] $e_EW[$I2_index][$j]\n";
    }
    close(OUT);
#    print "PASO\n"; exit;
#
# Input log in the rmodel format
# Hb-Mg

    open(TMP,">tmp.indices.log");
    print TMP "# rmodel version: 3.2.0   \n";
    print TMP "###############################################################################\n";
    print TMP "           0 # Model number\n";
    print TMP "           1 # number of 1st free parameter\n";
    print TMP "           2 # number of 2nd free parameter\n";
    print TMP "           1 # number of fixed value for IMF-slope\n";
    print TMP "           $I0_rmodel # number of 1st index\n";
    if ($type==0) {
	print TMP "           n # force logarithmic units?\n";
    } else {
	print TMP "           y # force logarithmic units?\n";
    }
    print TMP "           n # reverse axis in plots?\n";
    print TMP "           $I1_rmodel # number of 2nd index\n";
    print TMP "           n # force logarithmic units?\n";
    print TMP "           n # reverse axis in plots?\n";
    print TMP "           I # Menu option\n";
    print TMP "tmp.indices  # external ASCII file with data\n";
    print TMP "           y # estimate parameters for all objects?\n";
    print TMP "tmp.indices_rmodel # output ASCII file with results\n";
    print TMP "           k # pause between plots?\n";
#print TMP "           3 # method to find best polygon\n";
    print TMP "           2 # method to find best polygon\n";
    print TMP "         100 # no. of simulations\n";
    $time=time();
    print TMP "          $time # NSEED for random numbers\n";
    print TMP "rmodel_simul.out # output file for simulations\n";
    print TMP "           Q # Menu option\n";
    print TMP "END_of_rmodel.log\n";
    print TMP "\n";    
    close(TMP);

#
# File to run rmodel
#
    open(TMP,">rmodel.input");
    my $psname="myplots_".$I0_rmodel."_".$I1_rmodel.".ps/cps";
    print TMP "$psname\n";
    print TMP "tmp.indices.log\n";
    print TMP "Q\n";
    print TMP "N\n";
    close(TMP);

    $call="rmodel < rmodel.input > rmodel.JUNK";
    system($call);


    my $nj=0;
    my @age_best;
    my @met_best;
    
    open(TMP,"<tmp.indices_rmodel");
    while($line=<TMP>) {
	if ($line !~ "#") {
	    chop($line);
	    my @data=split(" ",$line);
	    $age_best[$nj]=$data[6];
	    $met_best[$nj]=0.019*10**($data[7]);
	    if (($age_best[$nj]<0)||($age_best[$nj]>14)) {
		$age_best[$nj]=$data[4];
		$met_best[$nj]=0.019*10**($data[5]);
	    }
	    if (($age_best[$nj]<0)||($age_best[$nj]>14)) {
		$age_best[$nj]=1000;
		$met_best[$nj]=1000;
	    }
	    $nj++;
	}
    }
    close(TMP);
    
    my @A_age;
    my @A_met;
    for ($j=0;$j<$n;$j++) {
	my $k=$j+1;
	my $num=$k;
	if ($k<10) {
	    $num="000".$k;
	} else {
	    if ($k<100) {
		$num="00".$k;
	    } else {
		if ($k<1000) {
		    $num="0".$k;
		}
	    }
	}
	my $tmpfile="rmodel_simul.out_obj".$num;
	$nc=0;
       
	open(TMP,"<$tmpfile");
	while($tmp=<TMP>) {
	    if ($tmp !~ "#") {
		chop($tmp);
		@data=split(" ",$tmp);
		$A_age[$j][$nc]=$data[1];
		$A_met[$j][$nc]=0.02*10**($data[0]);
		$nc++;
	    }
	}
	close(TMP);
    }

    my $NC=$nc;
    my @age,@met,@e_age,@e_met;

    for ($j=0;$j<$n;$j++) {
	my @tmp_age;
	my @tmp_met;
	for ($i=0;$i<$NC;$i++) {
	    $tmp_age[$i]=$A_age[$j][$i];
	    $tmp_met[$i]=$A_met[$j][$i];
	}
	$age[$j]=mean(@tmp_age);
	$met[$j]=mean(@tmp_met);
	if (($age[$j]<0)||($age[$j]>14)) {
	    $age[$j]=$age_best[$j];
	}
	if (($met[$j]<0)||($met[$j]>0.1)) {
	    $met[$j]=$met_best[$j];
	}
	

	$e_age[$j]=sigma(@tmp_age);
	$e_met[$j]=sigma(@tmp_met);
    }
    

##################################################################
#    print @e_met;
#    my $p1=\@age;
#    my $p2=\@e_age;
#    my $p3=\@met;
#    my $p4=\@e_met;
#
    my $pdl_age=pdl(@age);
    my $pdl_e_age=pdl(@e_age);
    my $pdl_met=pdl(@met);
    my $pdl_e_met=pdl(@e_met);    
    return $pdl_age,$pdl_e_age,$pdl_met,$pdl_e_met;
#    return (\@age,\@e_age,\@met,\@e_met);
#    return $p1,$p2,$p4,$p3;
}
