#!/usr/bin/perl
#
#


use Statistics::OLS;
use Math::FFT;
use Math::Stat;
use Math::Spline qw(spline linsearch binsearch);
use Math::Derivative qw(Derivative2);
use Math::Approx;
use Astro::FITS::CFITSIO qw( :longnames :constants );
use PDL;
use PDL::Fit::Polynomial; 
use PDL::Filter::Linear;
use PGPLOT;  # Load PGPLOT module
use PDL::Fit::Gaussian;


$F3D=$ENV{FIT3D_PATH}; $F3D=$ENV{FIT3D_PATH}; require("$F3D/scripts/my.pl");


#require "/home/sanchez/sda1/perl/MY/my.pl";

if ($#ARGV<0) {
    print "USE: clean_CS_rss.pl NAME\n";
    exit;
}

$name=$ARGV[0];
print "$name ";
#CS.NAME.RSS.fits.gz FIT.CS.NAME_FLUX.slice output.auto_ssp.CS.NAME.rss.out.rss.fits.gz CLEAN.CS.NAME.RSS.fits.gz CLEAN.FIT.CS.NAME_FLUX.slice GAS.CS.NAME.RSS.fits.gz\n";

$input_rss="CS.".$name.".RSS.fits.gz";
$input_slice="FIT.CS.".$name."_FLUX.slice";
$input_vel_slice="FIT.CS.".$name."_VEL.slice";
$input_disp_slice="FIT.CS.".$name."_DISP.slice";
$input_fit="output.auto_ssp.CS.".$name."rss.out.rss.fits.gz";
$output_rss="CLEAN.CS.".$name.".RSS.fits.gz";
$output_slice="CLEAN.FIT.CS.".$name."_FLUX.slice";
$output_EW="CLEAN.FIT.CS.".$name."_EW.slice";
$output_gas="gas.CS.".$name.".RSS.fits.gz";
$input_seg="seg_Ha.".$name.".fits.gz";
$output_seg="CLEAN.seg_Ha.CS.".$name.".fits.gz";
$output_vel_slice="CLEAN.FIT.CS.".$name."_VEL.slice";
#
# For the velocity we have to add the one derived for Halpha prior
# to correct the velocity map for the entire galaxy!
#

$input_vel_Ha="VEL.".$name.".slice";
open(VEL_Ha,"<$input_vel_Ha");
$n=0;
while($v_Ha=<VEL_Ha>) {
    chop($v_Ha);
    if ($v_Ha !~ "#") {
	@data=split(" ",$v_Ha);
	$VEL_Ha[$n]=$data[3];
	$n++;
    }
}
$mean_VEL_Ha=mean(@VEL_Ha);

$n=0;
$nh=0;
open(FH,"<$input_slice");
open(DISP,"<$input_disp_slice");
open(VEL,"<$input_vel_slice");
while($line=<FH>) {
    chop($line);
    $dline=<DISP>;
    chop($dline);
    $vline=<VEL>;
    chop($vline);
    if ($line !~ "#") {
	@data=split(" ",$line);
	$F[$n]=$data[3];
	$eF[$n]=$data[4];
	$LINE[$n]=$line;
	@data=split(" ",$dline);
	$D[$n]=$data[3];
	$vLINE[$n]=$vline;
	$n++;
    } else {
	$HEADER[$nh]=$line;
	$nh++;
    }
}
close(FH);

$input_pdl=rfits($input_rss);
$crval=$input_pdl->hdr->{CRVAL1};
$cdelt=$input_pdl->hdr->{CDELT1};
($nx,$ny)=$input_pdl->dims();
$output_pdl=$input_pdl;
$zero_pdl=zeroes($nx);
$input_pdl_fit=rfits($input_fit);
$input_pdl_gas=$input_pdl_fit->slice(":,:,(3)");
$input_pdl_nogas=$input_pdl_fit->slice(":,:,(5)");
$output_pdl_gas=$input_pdl_gas;
$h=$input_pdl->gethdr();
$output_pdl_gas->sethdr($h);

open(OUT,">$output_slice");
for ($i=0;$i<$nh;$i++) {
    print OUT "$HEADER[$i]\n";
}
$ne=0;
for ($i=3;$i<$nh;$i=$i+2) {
    $head=$HEADER[$i];
    ($junk,$junk2,$sec)=split(" ",$head);
    $cut="FLUX_";
    $sec =~ s/$cut//;
    $we[$ne]=$sec;
    $ne++;
}

open(OUT_VEL,">$output_vel_slice");
for ($i=0;$i<$nh;$i++) {
    $head=$HEADER[$i];
    $head =~ s/FLUX/VEL/g;
    print OUT_VEL "$head\n";
}

open(EW,">$output_EW");
for ($i=0;$i<$nh;$i++) {
    $head=$HEADER[$i];
    $head =~ s/FLUX/EW/g;
    print EW "$head\n";
}


#
#   Wrong SSP subtraction or vignetting effects
#
for ($j=0;$j<$ny;$j++) {
    #
    # Error in the residual
    #
    $nx1=int((3900-$crval)/$cdelt);
    $nx2=int((6800-$crval)/$cdelt);
    my $t=$input_pdl_gas->slice("$nx1:$nx2,($j)");
    ($mean,$sig,$med,$min,$max)=stats(abs($t));

    $n1=0;
    for ($i=0;$i<($nx2-$nx1);$i++) {
	$val=$t->at($i);
	if ($val>$med+$sig) {
	    $n1++;
	}
    }
    $f1[$j]=$n1/$nx;
    $nx1=int((6300-$crval)/$cdelt);
    $nx2=int((6500-$crval)/$cdelt);
    my $t=$input_pdl_gas->slice("$nx1:$nx2,($j)");
    ($mean,$sig,$med,$min,$max)=stats($t);
    $SIG[$j]=$sig*2.345*$D[$j];
    #
    # Flux density in the gas-free continuum
    #

}
$f1_med=median(@f1);
$f1_sig=sigma(@f1);
$f1_lim=3*($f1_med);

$n_bad=0;
@BAD;
for ($j=0;$j<$ny;$j++) {
#
#   Fraction of data==0
#
    $n0=0;
    for ($i=0;$i<$nx;$i++) {
	$val=$input_pdl->at($i,$j);
	if ($val==0) {
	    $n0++;
	}
    }
    $f0=$n0/$nx;
#
#   Wrong SSP subtraction or vignetting effects
#
    $k=$j+1;
#    print "$k $mean $sig $med $min $max\n"; <stdin>;
#    print "$k $f1[$j] $f1_lim\n"; <stdin>;
#    print "$k $F[$j] $eF[$j] $SIG[$j]\n"; <stdin>;

#    print "(($F[$j]<0)||($F[$j]>3*$F[0])||($f0>0.1)||($f1[$j]>$f1_lim)||($F[$j]<5*$SIG[$j]))\n";
    if (($F[$j]<=0)||($F[$j]>3*$F[0])||($f0>0.1)||($f1[$j]>$f1_lim)||($F[$j]<5*$SIG[$j])) {
	my $t=$output_pdl->slice(":,($j)");
	$t .= $zero_pdl;
	my $t=$output_pdl_gas->slice(":,($j)");
	$t .= $zero_pdl;
	$BAD[$n_bad]=$k;
	$n_bad++;
	print "BAD $k\n";
#	print " $k ";
    } else {
	my $line_now=$LINE[$j];
	$cut="1000.00";
	$new="   0.00";
	$cut="  -0.10";
	$new="   0.00";
	$line_now =~ s/$cut/$new/g;
	$k=$j+1;
	print "GOOD $k\n";
	print OUT "$line_now\n";

#################################################################
# EW START
#################
	$vline=$vLINE[$j];
	@vdata=split(" ",$vline);
	$line=$LINE[$j];
	@data=split(" ",$line);
	$nmod=$#vdata+1;
	
	$ne=0;
	$ID[$j]=$data[0];
	$X[$j]=$data[1];
	$Y[$j]=$data[2];


	printf(EW "%-2d %-7.4f %-7.4f ",$ID[$j],$X[$j],$Y[$j]);
	printf(OUT_VEL "%-2d %-7.4f %-7.4f ",$ID[$j],$X[$j],$Y[$j]);
	for ($k=3;$k<$nmod;$k=$k+2) {	
	    $ve[$ne]=$vdata[$k]+$VEL_Ha[$j]-$mean_VEL_Ha;
	    $eve[$ne]=$vdata[$k+1];
	    $fe[$ne]=$data[$k];
	    $efe[$ne]=$data[$k+1];
	if ($fe[$ne]==1000.0) {
	    $fe[$ne]=0;
	}
	    if ($fe[$ne]==10000.0) {
		$fe[$ne]=0;
	    }
	    if ($fe[$ne]==-0.1) {
		$fe[$ne]=0;
	    }
	    $Wobs=$we[$ne]*(1+$ve[$ne]/300000);
	    $nx1=int(($Wobs-50-$crval)/$cdelt);
	    $nx2=int(($Wobs+50-$crval)/$cdelt);
	    if ($nx1<0) {
		$nx1=0;
		$nx2=100;
	    }
	    if ($nx2>=$nx) {
		$nx1=$nx-101;
		$nx2=$nx-1;
	    }
	    my $t=$input_pdl_nogas->slice("$nx1:$nx2,($j)");
	    ($mean,$sig,$med,$min,$max)=stats($t);
	    if ($med!=0) {
		$EW[$ne][$j]=(-1)*$fe[$ne]/abs($med);
		$eEW[$ne][$j]=$efe[$ne]/abs($med)+$fe[$ne]*$sig/abs($med**2);
	    } else {
		$EW[$ne][$j]=0;
		$eEW[$ne][$j]=0;
	    }
	    printf(EW "%-7.4f %-7.4f ",$EW[$ne][$j],$eEW[$ne][$j]);
	    printf(OUT_VEL "%-7.4f %-7.4f ",$ve[$ne],$eve[$ne]);

	    $ne++;
	}
	print EW "\n";
	print OUT_VEL "\n";

	$med_ve=median(@ve);
#
# EW END
###################################################


    }
}
close(EW);
close(OUT);
print "\n";
if ($ny>0) {
    $f_bad=$n_bad/$ny;
} else {
    $f_bad=1;
}
print "$name $f_bad\n";
$output_pdl->wfits($output_rss);
$output_pdl_gas->wfits($output_gas);


$input_seg_pdl=rfits($input_seg);
$output_seg_pdl=$input_seg_pdl;
($nx,$ny)=$input_seg_pdl->dims;
for ($i=0;$i<$nx;$i++) {
    for ($j=0;$j<$ny;$j++) {
	$val=$input_seg_pdl->at($i,$j);
	for ($k=0;$k<$n_bad+1;$k++) {
	    if ($val==$BAD[$k]) {
		set($output_seg_pdl,$i,$j,0);
	    }
	}
    }
}
$output_seg_pdl->wfits($output_seg);

exit;
