#!/usr/bin/perl

use Statistics::OLS;
use Math::FFT;
use Math::Stat;
use Math::Spline qw(spline linsearch binsearch);
use Math::Derivative qw(Derivative2);

use Math::Approx;


use Astro::FITS::CFITSIO qw( :longnames :constants );
use PDL;

use PDL::Fit::Polynomial; 
use PDL::Filter::Linear;
use PGPLOT;  # Load PGPLOT module
use PDL::Fit::Gaussian;
use PDL::Image2D;
use  PDL::Fit::Linfit;



use PDL::Core;
use PDL::Basic;
use PDL::Exporter;
@ISA    = qw( PDL::Exporter );
use PDL::Options ':Func';
use PDL::Slatec; # For matinv()


$F3D=$ENV{FIT3D_PATH}; $F3D=$ENV{FIT3D_PATH}; require("$F3D/scripts/my.pl");


if ($#ARGV<0) {
    print "USE: sum_mass_age.pl NAME";
    exit;
}

$base_info="legacy/templates/GSD/BASE.gsd01";
open(FH,"<$base_info");
$line=<FH>; chop($line); $nbase=$line;
for ($i=0;$i<$nbase;$i++) {
    $line=<FH>;    chop($line); my @data=split(" ",$line);
    $mstar[$i]=$data[4];
}
close(FH);


$name=$ARGV[0];

$seg="cont_seg.".$name.".fits.gz";
$pdl_seg=rfits($seg);
($nx,$ny)=$pdl_seg->dims();

$flux_file="map.CS.".$name."_flux_ssp.fits.gz";
$pdl_flux=rfits($flux_file);
($nx,$ny)=$pdl_flux->dims();

$pdl_mask=ones($nx,$ny);

eval {
    $m_file="mask.".$name.".V.fits.gz";
    $pdl_mask=rfits($m_file);
};


$nl=0;
$file="auto_ssp.CS.".$name.".rss.out";
open(FH,"<$file");
while($line=<FH>) {
    if ($line !~ "#") {
	@data=split(",",$line);
	$med_flux[$nl]=$data[13];
	$e_med_flux[$nl]=$data[14];
	$a_redshift[$nl]=$data[7];
	$sum_flux=$sum_flux+$data[13];
	$nl++;
    }
}
close(FH);
$sum_flux=$sum_flux/$nl;
#print "$sum_flux\n";
$redshift=median(@a_redshift);
#$redshift=$redshift/$sum_flux;
#print "Z=$redshift\n";

@cosmo=cosm($redshift,71,0.27,0.73);
$ratio=3.08567758e24;
$modz=$cosmo[4];
#$DL=$cosmo[5]*$ratio;
#print "DL=$DL\n";
#$modz=
#
$DL=10**(($modz-25)/5)*$ratio;
$L=4*3.1416*($DL**2)*1;#/(1+$redshift);
$Factor=($L*1e-16)/3.826e33;



$file="coeffs_auto_ssp.CS.".$name.".rss.out";
$out_pack_csv="sum_mass_age.".$name.".pack.csv";
open(OUT,">$out_pack_csv");
print OUT "# (1) ID\n";
print OUT "# (2) File\n";
print OUT "# (3) Description\n";
print OUT "# (4) Type\n";
print OUT "# (5) units\n";
@a_age;
@a_met;
open(FH,"<$file");
$n=0;
$n_ini=0;

while(($line=<FH>)&&($n_ini<156)) {
    if ($line !~ "#") {
        @data=split(" ",$line);
        $n=$data[0];
        $ID[$n]=$data[0];
	$AGE[$n]=$data[1];
        $MET[$n]=$data[2];
	$COEFF[$n]=$data[3];
        $NORM[$n]=$data[4];
        $ML[$n]=$data[5];
        $AV[$n]=$data[6];
        $SUM[$n]=$SUM[$n]+$data[3];
	$SUM_MASS[$n]=$SUM_MASS[$n]+10**((-1)*$data[5]);
	$diff_age=1;
        for ($i=0;$i<$n;$i++) {
            if ($AGE[$n]==$AGE[$i]) {
                $diff_age=0;
            }
        }
        if ($diff_age==1) {
            $a_age[$n_age]=$AGE[$n];
            $n_age++;
        }

        $diff_met=1;
        for ($i=0;$i<$n;$i++) {
            if ($MET[$n]==$MET[$i]) {
		$diff_met=0;
            }
        }
        if ($diff_met==1) {
            $a_met[$n_met]=$MET[$n];
            $n_met++;
        }
        $n_ini++;
#       $n++;                                                                                                                                           
    }
}
close(FH);


print "# N=$n models\n";
print "# N.AGE=$n_age\n";
print "# N.MET=$n_met\n";

$pdl_ML=zeroes($nx,$ny);
$pdl_Mass=zeroes($nx,$ny);
$pdl_e_Mass=zeroes($nx,$ny);

#$Factor*$pdl_ML*$pdl_flux;

$i_n=1;
for ($i=0;$i<$n;$i++) {
    $norm_file="map.CS.".$name."_NORM_".$i."_ssp.fits.gz";
    $pdl_tmp=rfits($norm_file);
    $norm_e_file="map.CS.".$name."_eNORM_".$i."_ssp.fits.gz";
    $pdl_e_tmp=rfits($norm_e_file);
    $pdl_ML=$pdl_ML+$pdl_tmp*10**($ML[$i]);
    $pdl_Mass=$pdl_Mass+$pdl_tmp*10**($ML[$i])*$pdl_flux/$mstar[$i];
    $pdl_e_Mass=$pdl_e_Mass+$pdl_e_tmp*10**($ML[$i])*$pdl_flux/$mstar[$i];
    if ($i==0) {
	($nx,$ny)=$pdl_tmp->dims();
	$pdl_cube=zeroes($nx,$ny,$n);
	$pdl_e_cube=zeroes($nx,$ny,$n);
    }
    my $t=$pdl_cube->slice(":,:,($i)");
    $t .= $pdl_tmp;
    my $t=$pdl_e_cube->slice(":,:,($i)");
    $t .= $pdl_e_tmp;
    $file_name="map.CS.NAME_NORM_".$i."_ssp.fits.gz";
    print OUT "$i_n,$file_name,Luminosity Fraction for age-met $AGE[$i]-$MET[$i] SSP, flux, fraction\n";
    $i_n++;
}
$pdl_cube=$pdl_cube*$pdl_mask;
$pdl_e_cube=$pdl_e_cube*$pdl_mask;

$file_Av="map.CS.".$name."_Av_ssp.fits.gz";
$pdl_Av=rfits($file_Av);
#$pdl_dust_cor_log=0.4*$pdl_Av;

#$pdl_l_Mass_dust_corr=zeroes($nx,$ny);

#
$pdl_Mass=$pdl_Mass*$Factor;#+$pdl_tmp*10**($ML[$i])*$pdl_flux/$mstar[$i];
$pdl_e_Mass=$pdl_e_Mass*$Factor;#+$pdl_tmp*10**($ML[$i])*$pdl_flux/$mstar[$i];
$pdl_l_Mass=log($pdl_Mass)/log(10);
$pdl_l_Mass=$pdl_l_Mass*$pdl_mask;
$mass_file="map.CS.".$name."_Mass_ssp.fits.gz";
$pdl_l_Mass->wfits($mass_file);
$pdl_e_l_Mass=0.4*$pdl_e_Mass/$pdl_Mass;
$pdl_e_l_Mass=$pdl_e_l_Mass*$pdl_mask;
$mass_e_file="map.CS.".$name."_eMass_ssp.fits.gz";
$pdl_e_l_Mass->wfits($mass_e_file);


$mass_file="map.CS.".$name."_Mass_dust_cor_ssp.fits.gz";
$pdl_l_Mass_dust_corr=$pdl_l_Mass+0.4*$pdl_Av;
$pdl_l_Mass_dust_corr->wfits($mass_file);

$mass_file="map.CS.".$name."_ML_ssp.fits.gz";
$pdl_l_ML=log($pdl_ML)/log(10);
$pdl_l_ML->wfits($mass_file);

$Mass_sum=sum($pdl_Mass);
$e_Mass_sum=sum($pdl_e_Mass);
#$e_Mass_sum=sqrt(abs($e_Mass_sum));
$l_Mass=log10($Mass_sum);
$e_l_Mass=0.4*$e_Mass_sum/$Mass_sum;


#@dims=$pdl_l_Mass_dust_corr->dims;
#@dims=$pdl_l_Mass->dims;
#print "DIM = @dims\n";

$Mass_sum_dust_corr=0;
$pdl_l_Mass_dust_corr->inplace->setnantobad;
$pdl_l_Mass_dust_corr->inplace->setbadtoval(0);
$pdl_l_Mass->inplace->setnantobad;
$pdl_l_Mass->inplace->setbadtoval(0);

for ($i=0;$i<$nx;$i++) {
    for ($j=0;$j<$ny;$j++) {
#	print "$nx,$ny $i,$j\n";
	my $val=abs($pdl_l_Mass_dust_corr->at($i,$j,0));
	my $val2=abs($pdl_l_Mass->at($i,$j,0));
	if ($val>0) {
	    $Mass_sum_dust_corr=$Mass_sum_dust_corr+10**($val);
	} else {
	    if ($val2>0) {
		$Mass_sum_dust_corr=$Mass_sum_dust_corr+10**($val2);
	    }
	}
#	print "$val $Mass_sum_dust_corr\n"; <stdin>;
    }
}
$l_Mass_dust_corr=log10($Mass_sum_dust_corr);

print "# name, log10(Mass)\n";
print "Mass,$name,$l_Mass,$l_Mass_dust_corr,$e_l_Mass\n";



for ($i=0;$i<$n_age;$i++) {
    $age_now=$a_age[$i];
    $age_t=$age_now;
    if ($age_t<10) {
	$age_t="0".$age_t;
    }
    $age_file="map.CS.".$name."_".$age_t."_NORM_age.fits.gz";
    $age_e_file="map.CS.".$name."_".$age_t."_eNORM_age.fits.gz";
    $pdl_age=zeroes($nx,$ny);
    $pdl_e_age=zeroes($nx,$ny);
    for ($j=0;$j<$n;$j++) {
	if ($age_now==$AGE[$j]) {
	    my $t=$pdl_cube->slice(":,:,($j)");
	    $pdl_age=$pdl_age+$t;
	    my $t=$pdl_e_cube->slice(":,:,($j)");
	    $pdl_e_age=$pdl_e_age+$t**2;
	}
    }
    $pdl_age->wfits($age_file);
    $pdl_e_age->inplace->sqrt;
    $pdl_e_age->wfits($age_e_file);
    $file_name="map.CS.NAME_".$age_t."_NORM_age.fits.gz";
    print OUT "$i_n,$file_name,Luminosity Fraction for age $AGE[$i] SSP, flux, fraction\n";
    $i_n++;

}


for ($i=0;$i<$n_met;$i++) {
    $met_now=$a_met[$i];
    $met_file="map.CS.".$name."_".$met_now."_NORM_met.fits.gz";
    $pdl_met=zeroes($nx,$ny);
    $met_e_file="map.CS.".$name."_".$met_now."_eNORM_met.fits.gz";
    $pdl_e_met=zeroes($nx,$ny);
    for ($j=0;$j<$n;$j++) {
	if ($met_now==$MET[$j]) {
	    my $t=$pdl_cube->slice(":,:,($j)");
	    $pdl_met=$pdl_met+$t;
	    my $t=$pdl_e_cube->slice(":,:,($j)");
	    $pdl_e_met=$pdl_e_met+$t**2;
	}
    }    
    $pdl_met->wfits($met_file);
    $pdl_e_met->inplace->sqrt;
    $pdl_e_met->wfits($met_e_file);
    $file_name="map.CS.NAME_".$met_now."_NORM_met.fits.gz";
    print OUT "$i_n,$file_name,Luminosity Fraction for met $MET[$i] SSP, flux, fraction\n";
    $i_n++;
}

close(OUT);


exit;
